// *********************************************************
//
// agent2.cc      : Test agent #2 Main File
// author         : Fabio Silva
//
// $Id: agent2.cc,v 1.8 2001/04/05 21:28:24 fabio Exp $
//
// *********************************************************

#include "agent.hh"
#include <unistd.h>

MyReceive *mr;

NRSimpleAttributeFactory<char *> TargetAttr(NRAttribute::TARGET_KEY, NRAttribute::STRING_TYPE);
NRSimpleAttributeFactory<char *> AppStringAttr(APP_KEY2, NRAttribute::STRING_TYPE);
NRSimpleAttributeFactory<int> AppCounterAttr(APP_KEY3, NRAttribute::INT32_TYPE);

void MyReceive::recv(NRAttrVec *data, NR::handle my_handle)
{
   NRSimpleAttribute<int> *nrclass = NULL;

   nrclass = NRClassAttr.find(data);

   if (!nrclass){
      printf("Received a BAD packet !\n");
      return;
   }

   switch (nrclass->getVal()){

   case NRAttribute::INTEREST_CLASS:

      printf("Received an Interest message !\n");
      break;

   case NRAttribute::DISINTEREST_CLASS:

      printf("Received a Disinterest message !\n");
      break;

   default:

      printf("Received an unknown message (%d)!\n", nrclass->getVal());
      break;

   }
}

handle setupInterest(NR *dr)
{
   NRAttrVec attrs;
   
   attrs.push_back(NRClassAttr.make(NRAttribute::NE, NRAttribute::DATA_CLASS));
   attrs.push_back(NRScopeAttr.make(NRAttribute::IS, NRAttribute::NODE_LOCAL_SCOPE));
   attrs.push_back(TargetAttr.make(NRAttribute::EQ, "F117A"));
   attrs.push_back(LatitudeAttr.make(NRAttribute::IS, 60.00));
   attrs.push_back(LongitudeAttr.make(NRAttribute::IS, 54.00));
   
   handle h = dr->subscribe(&attrs, mr);

   ClearAttrs(&attrs);

   return h;
}

handle setupPublication(NR *dr)
{
   NRAttrVec attrs;

   attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::DATA_CLASS));
   attrs.push_back(LatitudeAttr.make(NRAttribute::IS, 60.00));
   attrs.push_back(LongitudeAttr.make(NRAttribute::IS, 54.00));
   attrs.push_back(TargetAttr.make(NRAttribute::IS, "F117A"));

   handle h = dr->publish(&attrs);

   ClearAttrs(&attrs);

   return h;
}

void main()
{
   NR *dr;
   int retval;
   int counter = 0;
   NRAttrVec data_attr;
   NRSimpleAttribute<int> *counterAttr;
   handle sendHandle;
   handle subscribeHandle;
   char input;

   dr = NR::createNR();
   mr = new MyReceive;

   subscribeHandle = setupInterest(dr);
   sendHandle = setupPublication(dr);

   data_attr.push_back(AppStringAttr.make(NRAttribute::IS, "Sensing is working !\0"));
   counterAttr = AppCounterAttr.make(NRAttribute::IS, counter);
   data_attr.push_back(counterAttr);
   
   // Main thread will send stuff
   while(1){
#ifdef INTERACTIVE
      printf("Now what ? ");
      input = getc(stdin);
      printf("\n");
      if (input == 'c'){
	 dr->unsubscribe(subscribeHandle);
	 exit(0);
      }
#else
      sleep(5);
#endif
      printf("Sending Data %d\n", counter);
      retval = dr->send(sendHandle, &data_attr);
      counter++;
      
      counterAttr->setVal(counter);
   }
}
