// *********************************************************
//
// logging.cc     : Log Application
// author         : Fabio Silva
//
// $Id: logging.cc,v 1.4 2000/12/08 23:53:05 fabio Exp $
//
// *********************************************************

#include "logging.hh"

LoggingApp *app;

char *msg_types[] = {"INTEREST", "POSITIVE REINFORCEMENT",
		     "NEGATIVE REINFORCEMENT", "DATA",
		     "FLAGGED DATA", "CONTROL", "REDIRECT"};

void MyFilterReceive::recv(Message *msg, handle h)
{
   app->recv(msg, h);
}

void LoggingApp::recv(Message *msg, handle h)
{
   if (h != filterHandle){
      printf("Error: LoggingApp::recv received message for handle %d when subscribing to handle %d !\n", h, filterHandle);
      return;
   }

   ProcessMessage(msg);

   ((DiffusionRouting *)dr)->sendMessageToNext(msg, h);
}

void LoggingApp::ProcessMessage(Message *msg)
{
   printf("Received a");

   if (msg->new_message)
      printf(" new ");
   else
      printf("n old ");

   if (msg->last_hop != LOCALHOST_ADDR)
      printf("%s message from node %d\n", msg_types[msg->msg_type],
	     msg->last_hop);
   else
      printf("%s message from agent %d\n", msg_types[msg->msg_type],
	     msg->source_port);
}

handle LoggingApp::setupFilter()
{
   NRAttrVec attrs;
   handle h;

   // This is a dummy attribute for filtering
   // It matches everything
   attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));

   h = ((DiffusionRouting *)dr)->addFilter(&attrs, LOGGING_APP_PRIORITY, fcb);

   ClearAttrs(&attrs);
   return h;
}

void LoggingApp::run()
{
   // Doesn't do anything
   while (1){
      sleep(10);
   }
}

LoggingApp::LoggingApp()
{
   // Create Diffusion Routing class
   dr = NR::createNR();
   fcb = new MyFilterReceive;
   
   // Set up the filter
   filterHandle = setupFilter();
   printf("Logging App subscribed to *, received handle %d\n", filterHandle);

   printf("Logging App initialized !\n");
}

void main()
{
   // Initialize and run the Logging App
   app = new LoggingApp;
   app->run();
}
