// *********************************************************
//
// simple_gradient.cc : Simple Gradient Application
// author             : Fabio Silva
//
// $Id: simple_gradient.cc,v 1.7 2001/03/23 21:02:49 fabio Exp $
//
// *********************************************************

#include "simple_gradient.hh"

GradientApp *app;

void MyFilterReceive::recv(Message *msg, handle h)
{
   app->recv(msg, h);
}

int MyTimerReceive::recv(handle hdl, void *p)
{
   return app->ProcessTimers(hdl, p);
}

void MyTimerReceive::del(void *p)
{
  TimerType *timer;
  NRAttrVec *attrs;
  Message *msg;

  timer = (TimerType *) p;

  switch (timer->which_timer){

  case SUBSCRIPTION_TIMER:

    attrs = ((NRAttrVec *)timer->param);

    if (attrs){
      ClearAttrs(attrs);
      delete attrs;
    }

    break;

  case INTEREST_TIMER:

    msg = ((Message *)timer->param);

    if (msg)
      delete msg;

    break;
  }

  delete timer;
}

int GradientApp::ProcessTimers(handle hdl, void *p)
{
  TimerType *timer;
  NRAttrVec *attrs;
  Message *msg;
  int timeout = 0;

  timer = (TimerType *) p;
   
  switch (timer->which_timer){
  case GRADIENT_TIMER:

    GradientTimeout();

    break;

  case SUBSCRIPTION_TIMER:

    attrs = ((NRAttrVec *)timer->param);

    timeout = SubscriptionTimeout(attrs);

    break;

  case INTEREST_TIMER:

    msg = ((Message *)timer->param);

    InterestTimeout(msg);

    // Cancel Timer
    timeout = -1;

    break;

  default:

    printf("Error: GradientApp::ProcessTimers received unknown timer %d !\n", timer->which_timer);

    break;
  }

  return timeout;
}

void GradientApp::InterestTimeout(Message *msg)
{
   printf("Interest Timeout !\n");

   msg->last_hop = LOCALHOST_ADDR;
   msg->next_hop = BROADCAST_ADDR;
   
   ((DiffusionRouting *)dr)->sendMessage(msg, filterHandle);
}

void GradientApp::GradientTimeout()
{
   RoutingList::iterator Ritr;
   AgentsList::iterator Aitr;
   Routing_Entry *Rentry;
   Agents_Entry *Aentry, *Hentry;
   struct timeval tmv;

   printf("Gradient Timeout !\n");
   
   gettimeofday(&tmv, NULL);

   Ritr = routing_list.begin();

   while (Ritr != routing_list.end()){
      Rentry = *Ritr;
      Hentry = NULL;
      
      // Step 1: Delete expired gradients and figure out the
      // prefered output gradient
      Aitr = Rentry->active.begin();
      while (Aitr != Rentry->active.end()){
	 Aentry = *Aitr;
	 if (tmv.tv_sec > (Aentry->tv.tv_sec + GRADIENT_TIMEOUT)){
	    printf("Deleting Gradient to node %d !\n", Aentry->node_addr);
	    Aitr = Rentry->active.erase(Aitr);
	    delete Aentry;
	 }
	 else{
	    if (!Hentry)
	       Hentry = Aentry;
	    else
	       if (Aentry->num_new_messages > Hentry->num_new_messages)
		  Hentry = Aentry;
	    Aitr++;
	 }
      }

      // Step 2: Clear number of new messages for all gradients,
      //         except for the prefered gradient, who gets '1'
      //         in order to continue getting data messages
      if (Rentry->active.size() > 0){
	 Aitr = Rentry->active.begin();
	 while (Aitr != Rentry->active.end()){
	    Aentry = *Aitr;
	    Aentry->num_new_messages = 0;
	    Aitr++;
	 }
	 Hentry->num_new_messages = 1;
      }

      Aitr = Rentry->agents.begin();
      while (Aitr != Rentry->agents.end()){
	 Aentry = *Aitr;
	 if (tmv.tv_sec > (Aentry->tv.tv_sec + GRADIENT_TIMEOUT)){
	    printf("Deleting Gradient to agent %d !\n", Aentry->port);
	    Aitr = Rentry->agents.erase(Aitr);
	    delete Aentry;
	 }
	 else{
	    Aitr++;
	 }
      }
      
      // Remove the Routing Entry if no gradients and no agents
      if ((Rentry->active.size() == 0) &&
	  (Rentry->agents.size() == 0)){
	 // Deleting Routing Entry
	 printf("Nothing left for this data type, cleaning up !\n");
	 Ritr = routing_list.erase(Ritr);
	 delete Rentry;
      }
      else{
	 Ritr++;
      }
   }
}

int GradientApp::SubscriptionTimeout(NRAttrVec *attrs)
{
   Routing_Entry *entry;
   AttributeList::iterator itr;
   Attribute_Entry *ae;
   struct timeval tmv;

   printf("Subscription Timeout !\n");

   gettimeofday(&tmv, NULL);

   // Find the correct Routing Entry
   entry = FindPerfectMatch(attrs);

   if (entry){
      // Step 1: Check Timeouts
      
      itr = entry->attr_list.begin();

      while (itr != entry->attr_list.end()){
	 ae = *itr;
	 if (tmv.tv_sec > (ae->tv.tv_sec + SUBSCRIPTION_TIMEOUT)){
	       SendDisinterest(ae->attrs, entry);
	       itr = entry->attr_list.erase(itr);
	       delete ae;
	 }
	 else{
	    itr++;
	 }
      }
   }
   else{
      printf("Warning: GradientApp::SubscriptionTimeout could't find routing entry - maybe destroyed by Gradient Timeout ?\n");

      // Cancel Timer
      return -1;
   }

   // Keep Timer
   return 0;
}

void GradientApp::DeleteRoutingEntry(Routing_Entry *entry)
{
   RoutingList::iterator itr;
   Routing_Entry *e;

   for (itr = routing_list.begin(); itr != routing_list.end(); ++itr){
      e = *itr;
      if (e == entry){
	 itr = routing_list.erase(itr);
	 delete entry;
	 return;
      }
   }
   printf("Error: GradientApp::DeleteRoutingEntry could not find entry to delete !\n");
}

Routing_Entry * GradientApp::FindRoutingEntry(NRAttrVec *attrs, RoutingList::iterator start, RoutingList::iterator *place)
{
   RoutingList::iterator itr;
   Routing_Entry *entry;

   for (itr = start; itr != routing_list.end(); ++itr){
      entry = *itr;
      if (MatchAttrs(entry->attrs, attrs)){
	 *place = itr;
	 return entry;
      }
   }
   return NULL;
}

Routing_Entry * GradientApp::FindPerfectMatch(NRAttrVec *attrs)
{
   RoutingList::iterator itr;
   Routing_Entry *entry;

   for (itr = routing_list.begin(); itr != routing_list.end(); ++itr){
      entry = *itr;
      if (PerfectMatch(entry->attrs, attrs))
	 return entry;
   }
   return NULL;
}

Attribute_Entry * GradientApp::FindMatchingSubscription(Routing_Entry *entry,
							NRAttrVec *attrs)
{
   AttributeList::iterator itr;
   Attribute_Entry *ae;

   for (itr = entry->attr_list.begin(); itr != entry->attr_list.end(); ++itr){
      ae = *itr;
      if (PerfectMatch(ae->attrs, attrs))
	 return ae;
   }
   return NULL;
}

void GradientApp::UpdateGradient(Routing_Entry *entry, int32_t last_hop,
				 bool new_message)
{
   AgentsList::iterator itr;
   Agents_Entry *ae;

   for (itr = entry->active.begin(); itr != entry->active.end(); ++itr){
      ae = *itr;
      if (ae->node_addr == last_hop){
	 gettimeofday(&(ae->tv), NULL);
	 if (new_message)
	    ae->num_new_messages++;
	 return;
      }
   }

   // We need to add a new gradient
   ae = new Agents_Entry;
   ae->node_addr = last_hop;
   ae->port = DIFFUSION_PORT;
   if (new_message)
      ae->num_new_messages++;
   
   entry->active.push_back(ae);
}

void GradientApp::UpdateAgent(Routing_Entry *entry, u_int16_t source_port)
{
   AgentsList::iterator itr;
   Agents_Entry *ae;

   for (itr = entry->agents.begin(); itr != entry->agents.end(); ++itr){
      ae = *itr;
      if (ae->port == source_port){
	 // We already have this guy
	 gettimeofday(&(ae->tv), NULL);
	 return;
      }
   }

   // This is a new Agent
   ae = new Agents_Entry;
   ae->node_addr = LOCALHOST_ADDR;
   ae->port = source_port;

   entry->agents.push_back(ae);
}

void GradientApp::ForwardData(Message *msg, Routing_Entry *entry)
{
   AgentsList::iterator itr;
   Agents_Entry *ae, *Aentry;
   Message *mymsg;

   mymsg = CopyMessage(msg);

   // Forward DATA to local agents
   for (itr = entry->agents.begin(); itr != entry->agents.end(); ++itr){
      ae = *itr;
      
      mymsg->next_hop = LOCALHOST_ADDR;
      ((DiffusionRouting *)dr)->sendMessage(mymsg, filterHandle, ae->port);
   }

   // Forward DATA to output gradient
   itr = entry->active.begin();
   ae = NULL;
   while (itr != entry->active.end()){
      Aentry = *itr;
      if (!ae)
	 ae = Aentry;
      else
	 if (Aentry->num_new_messages > ae->num_new_messages)
	    ae = Aentry;
      itr++;
   }

   if (ae){
      if (ae->num_new_messages > 0){
	 mymsg->next_hop = ae->node_addr;
	 ((DiffusionRouting *)dr)->sendMessage(mymsg, filterHandle);
      }
   }

   delete mymsg;
}

void GradientApp::SendInterest(NRAttrVec *attrs, Routing_Entry *entry)
{
   AgentsList::iterator itr;
   Agents_Entry *ae;

   Message *msg = new Message(DIFFUSION_VERSION, INTEREST, 0, 0,
			      attrs->size(), 0, 0, LOCALHOST_ADDR,
			      LOCALHOST_ADDR);

   msg->msg_attr_vec = CopyAttrs(attrs);

   for (itr = entry->agents.begin(); itr != entry->agents.end(); ++itr){
      ae = *itr;

      ((DiffusionRouting *)dr)->sendMessage(msg, filterHandle, ae->port);
   }

   delete msg;
}

void GradientApp::SendDisinterest(NRAttrVec *attrs, Routing_Entry *entry)
{
   NRAttrVec *newAttrs;
   NRSimpleAttribute<int> *nrclass = NULL;

   newAttrs = CopyAttrs(attrs);

   nrclass = NRClassAttr.find(newAttrs);
   if (!nrclass){
      printf("Error: GradientApp::SendDisinterest couldn't find the class attribute !\n");
      ClearAttrs(newAttrs);
      delete newAttrs;
      return;
   }

   // Change the class_key value
   nrclass->setVal(NRAttribute::DISINTEREST_CLASS);

   SendInterest(newAttrs, entry);
   
   ClearAttrs(newAttrs);
   delete newAttrs;
}

void GradientApp::recv(Message *msg, handle h)
{
   if (h != filterHandle){
      printf("Error: GradientApp::recv received message for handle %d when subscribing to handle %d !\n", h, filterHandle);
      return;
   }

   if (msg->new_message == 1)
      ProcessNewMessage(msg);
   else
      ProcessOldMessage(msg);
}

void GradientApp::ProcessOldMessage(Message *msg)
{
   Routing_Entry *entry;

   switch (msg->msg_type){

   case INTEREST:

      printf("Received Old Interest !\n");

      if (msg->last_hop == LOCALHOST_ADDR){
	 // Old interest should not come from local agent
	 printf("Warning: Old Interest from local agent !\n");
	 break;
      }

      // Get the routing entry for these attrs      
      entry = FindPerfectMatch(msg->msg_attr_vec);
      if (entry)
	 UpdateGradient(entry, msg->last_hop, false);

      break;
      
   case DATA: 
      
      // Just drop it
      printf("Received Old Data. Loop detected !\n");
      
      break;
      
   default: 
      
      break;
   }
}

void GradientApp::ProcessNewMessage(Message *msg)
{
   NRSimpleAttribute<int> *nrclass = NULL;
   NRSimpleAttribute<int> *nrscope = NULL;
   Routing_Entry *entry;
   Attribute_Entry *ae;
   RoutingList::iterator place;
   NRAttrVec::iterator itr;
   bool new_data_type = false;
   
   switch (msg->msg_type){
      
   case INTEREST:
      
      printf("Received Interest !\n");
      
      // Step 1: Look for the same data type
      entry = FindPerfectMatch(msg->msg_attr_vec);

      if (!entry){
	 // Create a new routing entry for this data type
	 entry = new Routing_Entry;
	 entry->attrs = CopyAttrs(msg->msg_attr_vec);
	 routing_list.push_back(entry);
	 new_data_type = true;
      }
	 
      if (msg->last_hop == LOCALHOST_ADDR){
	 // From local agent
	 UpdateAgent(entry, msg->source_port);
      }
      else{
	 // From outside, we just add the new gradient
	 UpdateGradient(entry, msg->last_hop, true);
      }

      nrclass = NRClassAttr.find(msg->msg_attr_vec);
      nrscope = NRScopeAttr.find(msg->msg_attr_vec);

      if ((nrclass->getVal() == NRAttribute::INTEREST_CLASS) &&
	  (nrclass->getOp() == NRAttribute::IS) &&
	  (nrscope->getVal() == NRAttribute::GLOBAL_SCOPE)){
	 // Forward this interest

	 TimerType *timer;
	 
	 timer = new TimerType(INTEREST_TIMER);
	 timer->param = (void *) CopyMessage(msg);

	 ((DiffusionRouting *)dr)->addTimer(INTEREST_FORWARD_DELAY +
					    (int) ((INTEREST_FORWARD_JITTER * (rand() * 1.0 / RAND_MAX)) - (INTEREST_FORWARD_JITTER / 2)),
					    (void *) timer, tcb);

      }
      else{
	 if ((nrclass->getOp() != NRAttribute::IS) &&
	     (new_data_type)){

	    TimerType *timer;
	    
	    timer = new TimerType(SUBSCRIPTION_TIMER);
	    timer->param = (void *) CopyAttrs(msg->msg_attr_vec);
	    
	    ((DiffusionRouting *)dr)->addTimer(SUBSCRIPTION_DELAY +
					       (int) (SUBSCRIPTION_DELAY * (rand() * 1.0 / RAND_MAX)),
					       (void *) timer, tcb);
	 }

	 // Subscriptions don't have to match other subscriptions
	 break;
      }
      
      // Step 2: Match other routing tables
      place = routing_list.begin();
      entry = FindRoutingEntry(msg->msg_attr_vec, place, &place);

      while (entry){
	 // Got a match
	 ae = FindMatchingSubscription(entry, msg->msg_attr_vec);

	 // Do we already have this subscription
	 if (ae){
	    gettimeofday(&(ae->tv), NULL);
	 }
	 else{
	    ae = new Attribute_Entry;
	    ae->attrs = CopyAttrs(msg->msg_attr_vec);
	    entry->attr_list.push_back(ae);
	    SendInterest(ae->attrs, entry);
	 }
	 // Move to the next Routing_Entry
	 place++;

	 entry = FindRoutingEntry(msg->msg_attr_vec, place, &place);
      }

      break;
      
   case DATA:

      printf("Received Data !\n");

      // Find the correct routing entry
      place = routing_list.begin();
      entry = FindRoutingEntry(msg->msg_attr_vec, place, &place);

      while (entry){
	 ForwardData(msg, entry);
	 place++;
	 entry = FindRoutingEntry(msg->msg_attr_vec, place, &place);
      }
      
      break;
      
   default:
      
      break;
   }
}

handle GradientApp::setupFilter()
{
   NRAttrVec attrs;
   handle h;

   // This is a dummy attribute for filtering
   // It matches everything
   attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));

   h = ((DiffusionRouting *)dr)->addFilter(&attrs, GRADIENT_APP_PRIORITY, fcb);

   ClearAttrs(&attrs);
   return h;
}

void GradientApp::run()
{
   // Doesn't do anything
   while (1){
      sleep(10);
   }
}

GradientApp::GradientApp()
{
   // Create Diffusion Routing class
   dr = NR::createNR();
   fcb = new MyFilterReceive;
   tcb = new MyTimerReceive;
   
   // Set up the filter
   filterHandle = setupFilter();
   printf("Gradient App subscribed to *, received handle %d\n", filterHandle);

   // Add the Gradient Timer
   TimerType *timer;

   timer = new TimerType(GRADIENT_TIMER);
   ((DiffusionRouting *)dr)->addTimer(GRADIENT_DELAY, (void *) timer, tcb);

   printf("Gradient App initialized !\n");
}

void main()
{
   // Initialize and run the Gradient App
   app = new GradientApp;
   app->run();
}
