// *********************************************************
//
// simple_gradient.hh : Gradient Application Include File
// author             : Fabio Silva
//
// $Id: simple_gradient.hh,v 1.4 2001/03/23 21:02:49 fabio Exp $
//
// *********************************************************

#include <stdlib.h>
#include <stdio.h>
#include "dr.hh"

#define GRADIENT_APP_PRIORITY 2

class Attribute_Entry;
class Routing_Entry;
class Agents_Entry;

typedef list<Attribute_Entry *> AttributeList;
typedef list<Routing_Entry *> RoutingList;
typedef list<Agents_Entry *> AgentsList;

class Agents_Entry : public Agent_Entry {
public:
  int32_t num_new_messages;

  Agents_Entry() : Agent_Entry() {
    num_new_messages = 0;
  };
};

class MyFilterReceive : public FilterCallback {
public:
   void recv(Message *msg, handle h);
};

class MyTimerReceive : public TimerCallbacks {
public:
  int recv(handle hdl, void *p);
  void del(void *p);
};

class TimerType {
public:
  int which_timer;
  void *param;

  TimerType(int _which_timer) : which_timer(_which_timer)
  {
    param = NULL;
  };

  ~TimerType() {};
};

class Attribute_Entry {
public:
   NRAttrVec       *attrs;
   struct timeval  tv;

   Attribute_Entry() {
      attrs        = NULL;
      gettimeofday(&tv, NULL);
   };

   ~Attribute_Entry() {
      ClearAttrs(attrs);
      delete attrs;
   };
};

class Routing_Entry {
public:
   AgentsList      active;
   AgentsList      agents;
   NRAttrVec       *attrs;
   AttributeList   attr_list;
   
   Routing_Entry() {};

   ~Routing_Entry() {
      AttributeList::iterator ATitr;
      AgentsList::iterator AGitr;
      
      // Clear Attributes
      ClearAttrs(attrs);
      delete attrs;
      
      ATitr = attr_list.begin();

      while (ATitr != attr_list.end()){
	 delete (*ATitr);
	 ATitr++;
      }
      attr_list.clear();

      AGitr = active.begin();
      
      while (AGitr != active.end()){
	 delete (*AGitr);
	 AGitr++;
      }
      active.clear();

      AGitr = agents.begin();
      
      while (AGitr != agents.end()){
	 delete (*AGitr);
	 AGitr++;
      }
      agents.clear();
   };
};

class GradientApp {
public:
   GradientApp();
   void run();

   void recv(Message *msg, handle h);
   int ProcessTimers(handle hdl, void *p);

protected:

   // General Variables
   NR *dr;
   handle filterHandle;

   // Receive Callback for the filter
   MyFilterReceive *fcb;
   MyTimerReceive *tcb;

   // List of all known datatypes
   RoutingList     routing_list;

   // Setup the filter
   handle setupFilter();

   // Matching functions
   Routing_Entry * FindPerfectMatch(NRAttrVec *attrs);
   void DeleteRoutingEntry(Routing_Entry *entry);
   Routing_Entry * FindRoutingEntry(NRAttrVec *attrs, RoutingList::iterator start, RoutingList::iterator *place);
   Attribute_Entry * FindMatchingSubscription(Routing_Entry *entry, NRAttrVec *attrs);

   // Data structure management
   void UpdateGradient(Routing_Entry *entry, int32_t last_hop, bool new_message);
   void UpdateAgent(Routing_Entry *entry, u_int16_t source_port);

   // Message forwarding functions
   void SendInterest(NRAttrVec *attrs, Routing_Entry *entry);
   void SendDisinterest(NRAttrVec *attrs, Routing_Entry *entry);
   void ForwardData(Message *msg, Routing_Entry *entry);

   // Message Processing functions
   void ProcessOldMessage(Message *msg);
   void ProcessNewMessage(Message *msg);

   // Timers
   void InterestTimeout(Message *msg);
   void GradientTimeout();
   int SubscriptionTimeout(NRAttrVec *attrs);
};
