// *********************************************************
//
// tag.cc         : Tag Application
// author         : Fabio Silva
//
// $Id: tag.cc,v 1.5 2001/02/26 23:17:59 fabio Exp $
//
// *********************************************************

#include "tag.hh"

TagApp *app;

void MyFilterReceive::recv(Message *msg, handle h)
{
  app->recv(msg, h);
}

void TagApp::recv(Message *msg, handle h)
{
  if (h != filterHandle){
    fprintf(stderr, "Error: TagApp::recv received message for handle %d when subscribing to handle %d !\n", h, filterHandle);
    return;
  }

  ProcessMessage(msg);

  ((DiffusionRouting *)dr)->sendMessageToNext(msg, h);
}

void TagApp::ProcessMessage(Message *msg)
{
  char *original_route;
  char *new_route;
  int len, total_len;
  NRSimpleAttribute<char *> *route = NULL;

  // Can't do anything if node id unknown
  if (!id)
    return;

  route = RouteAttr.find(msg->msg_attr_vec);
  if (!route){
    fprintf(stderr, "Error: Can't find the route attribute !\n");
    return;
  }

  original_route = route->getVal();
  len = strlen(original_route);

  if (len == 0){
    total_len = strlen(id);
    // Route is empty, need to allocate memory
    // for our id and the terminating '\0'
    new_route = new char[(total_len + 1)];
    strcpy(new_route, id);
    if (new_route[total_len] != '\0'){
      printf("Warning: String must end with NULL !\n");
    }
  }
  else{
    // Route already exists. We need to allocate
    // memory for the current route + ':' + our
    // id + the terminating '\0'
    total_len = len + strlen(id) + 1;
    new_route = new char[(total_len + 1)];
    strcpy(new_route, original_route);
    new_route[len] = ':';
    strcpy(&new_route[len+1], id);
    if (new_route[total_len] != '\0'){
      printf("Warning: String must end with NULL !\n");
    }
  }

  // Debug
  fprintf(stderr, "Tag App: Original route : %s\n", original_route);
  fprintf(stderr, "Tag App: New route : %s\n", new_route);

  route->setVal(new_route);

  // Free memory
  delete [] new_route;
}

handle TagApp::setupFilter()
{
  NRAttrVec attrs;
  handle h;

  // This is a dummy attribute for filtering
  // It matches everything
  attrs.push_back(RouteAttr.make(NRAttribute::EQ_ANY, ""));

  h = ((DiffusionRouting *)dr)->addFilter(&attrs, TAG_APP_PRIORITY, fcb);

  ClearAttrs(&attrs);
  return h;
}

void TagApp::run()
{
  // Doesn't do anything
  while (1){
    sleep(10);
  }
}

void TagApp::getNodeId()
{
  // Future implementation for the inter-module API
}

TagApp::TagApp()
{
  char *id_env = NULL;
  char buffer[BUFFER_SIZE];
  int flag;

  id = NULL;
  node_id = 0;

  // Create Diffusion Routing class
  dr = NR::createNR();
  fcb = new MyFilterReceive;

  // Try to figure out the node ID
  id_env = getenv("scadds_addr");
  if (id_env){
    node_id = atoi(id_env);
    flag = snprintf(&buffer[0], BUFFER_SIZE, "%d", node_id);
    if (flag == -1 || flag == BUFFER_SIZE){
      fprintf(stderr, "Error: Buffer too small !\n");
      exit(-1);
    }
    id = strdup(&buffer[0]);
  }
  else{
    getNodeId();
  }
  // Set up the filter
  filterHandle = setupFilter();
  fprintf(stderr, "Tag App subscribed to *, received handle %d\n", filterHandle);
  fprintf(stderr, "Tag App initialized !\n");
}

void main()
{
  // Initialize and run the Tag App
  app = new TagApp;
  app->run();
}
