// *********************************************************
//
// dgram.cc      : Basic Socket Functions
// authors       : Chalermek Intanagonwiwat and Fabio Silva
//
// $Id: dgram.cc,v 1.7 2001/03/20 23:02:20 fabio Exp $
//
// *********************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>

#include "dgram.hh"

struct hostent * GetThisHostEntWired() 
{
  struct hostent *hEnt = new struct hostent;
  struct hostent *temp = NULL;

  if ((temp = gethostbyname("localhost")) == NULL){
    perror("Can't get host entry.\n");
    exit(-1);
  }

  memcpy(hEnt, temp, sizeof(struct hostent));
  return hEnt;
}

struct hostent * GetThisHostEnt() 
{
  struct hostent *hEnt = new struct hostent;
  struct hostent *temp = NULL;

  if ((temp = gethostbyname("localhost")) == NULL){
    perror("Can't get host entry.\n");
    exit(-1);
  }

  memcpy(hEnt, temp, sizeof(struct hostent));
  return hEnt;
}

struct socket_info * CreateSocket(struct hostent *h_entPtr, u_int16_t port)
{
  int fd;
  struct socket_info *skt_info = new struct socket_info;

  if (h_entPtr == NULL){
    perror("hostent is NULL. Can't create socket.\n");
    exit(-1);
    return NULL;
  }

  if (skt_info == NULL){
    perror("Can't allocate struct socket_info.\n");
    exit(-1);
    return NULL;
  }

  fd = socket(AF_INET, SOCK_DGRAM, 0);
  if (fd < 0){
    perror("Can't create socket.\n");
    delete skt_info;
    exit(-1);
    return NULL;
  }

  skt_info->skt_fd = fd;

  // Clean things out first.
  bzero((char *) &(skt_info->skt_addr_inet), sizeof(skt_info->skt_addr_inet));

  // Configure socket
  skt_info->skt_addr_inet.sin_family = AF_INET;
  skt_info->skt_addr_inet.sin_port = htons(port);
  bcopy(h_entPtr->h_addr, (char *) &(skt_info->skt_addr_inet.sin_addr), 
	h_entPtr->h_length );

  return skt_info;
}

void RegisterSocket(struct socket_info *skt_info)
{
  if (bind(skt_info->skt_fd, (struct sockaddr *) &(skt_info->skt_addr_inet), 
	   sizeof(skt_info->skt_addr_inet)) < 0){
    printf("Can't bind to socket.\n");
    close(skt_info->skt_fd);
    delete (skt_info);
    exit(-1);
  }
}

struct socket_info * OpenIncomingSocket(u_int16_t port)
{
  int fd;
  struct socket_info *skt_info = new struct socket_info;

  if (skt_info == NULL){
    printf("Can't create socket_info\n");
    exit(-1);
  }

  fd = socket(AF_INET, SOCK_DGRAM, 0);

  skt_info->skt_fd = fd;

  bzero((char *) &(skt_info->skt_addr_inet), sizeof(skt_info->skt_addr_inet));

  skt_info->skt_addr_inet.sin_family = AF_INET;
  skt_info->skt_addr_inet.sin_port = htons(port);
  skt_info->skt_addr_inet.sin_addr.s_addr = htonl(INADDR_ANY);

  if (bind(skt_info->skt_fd, (struct sockaddr *) &(skt_info->skt_addr_inet),
	   sizeof(skt_info->skt_addr_inet)) < 0){
    printf("Can't bind to incoming socket\n");
    exit(-1);
  }

  return skt_info;
}
