// *********************************************************
// 
// diffusion.hh  : Diffusion definitions
// authors       : Chalermek Intanagonwiwat and Fabio Silva
//
// $Id: diffusion.hh,v 1.23 2001/05/22 22:13:31 fabio Exp $
//
// *********************************************************

#ifndef diffusion_hh
#define diffusion_hh

extern "C"{
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <signal.h>
#include <float.h>
#include <string.h>
#include <stdlib.h>
}

#include "utils.hh"
#include "events.hh"
#include "message.hh"
#include "filter.hh"
#include "config.hh"
#include "agent.hh"

#ifdef WIRED
#include "replicator.hh"
#endif

#ifdef STATS
#include "stats.hh"
#endif

#ifndef WIRED
#    ifdef USE_RPC
#include "drivers/rpc_stats.hh"
#    endif
#endif

Packet DupPacket(Packet pkt);  // Allocate and copy pkt

#define SKT_INFO(x)       x->skt_info

class DiffusionAgent;
class Neighbor_Entry;

typedef list<Neighbor_Entry *> NeighborList;
typedef list<Tcl_HashEntry *> HashList;

class Hash_Entry {
public:
   bool        dummy;

   Hash_Entry() { 
      dummy  = false;
   }
};

class Neighbor_Entry {
public:
  int32_t id;
  struct timeval tmv;

  Neighbor_Entry(int _id) : id(_id)
  {
    gettimeofday(&tmv, NULL);
  }
};

class DiffusionAgent {
public:
  DiffusionAgent(int argc, char **argv);
  void DiffusionAgent::usage();
  void run();
  void TimetoStop();

protected:
  float lon;
  float lat;

  // File descriptor of the radio if we are using the kernel module
  int kernel_radio;

  // Socket variables
  struct hostent *host_ent;
  struct socket_info *in_sock;
  unsigned int addrlen;                       // Internet address length

#ifdef WIRED
  DiffusionWired *wired;
#endif

#ifdef STATS
  DiffusionStats *stats;
#  ifndef WIRED
#     ifdef USE_RPC
  RPCStats *rpcstats;
#     endif
#  endif

#endif

  // Packet variables
  int in_buffer[MAX_PKT_SIZE / sizeof(int)];  // Buffer for incoming packets
  struct hdr_diff *in_packet_dfh;             // Header of incoming packets
  int out_buffer[MAX_PKT_SIZE / sizeof(int)]; // Buffer for outgoing packets
  struct hdr_diff *out_packet_dfh;

  // Ids and counters
  int32_t myId;
  int pkt_count;
  int rdm_id;

  // Configuration options
  char *config_file;
  int debug_level;

  // Lists
  NeighborList my_neighbors;
  FilterList filter_list;
  AgentList agent_list;
  HashList hash_list;

  // Data structures
  eventQueue *eq;
  Tcl_HashTable htable;

  // Methods start here

  // Device-Independent send
  void dvi_send(caddr_t data, int length);

  // Receive and packet processing
  void add_to_set(int socket, fd_set *set, int *max);
  void recv();

  // Hashing functions
  Hash_Entry *GetHash(unsigned int, unsigned int);
  void PutHash(unsigned int, unsigned int);

  // Neighbors functions
  void UpdateNeighbors(int id);
  void UpdateAgents(Message *msg);

  // Message functions
  void ProcessMessage(Message *msg);
  void ProcessControlMessage(Message *msg);
  void LogControlMessage(Message *msg, int command, int param1, int param2);
  bool RestoreOriginalHeader(Message *msg);

  // Filter functions
  FilterList * GetFilterList(NRAttrVec *attrs);
  Filter_Entry * FindFilter(int16_t handle, u_int16_t agent);
  Filter_Entry * DeleteFilter(int16_t handle, u_int16_t agent);
  void AddFilter(NRAttrVec *attrs, u_int16_t agent, int16_t handle, u_int16_t priority);
  FilterList::iterator FindMatchingFilter(NRAttrVec *attrs, FilterList::iterator itr);

  // Send messages to modules
  void ForwardMessage(Message *msg, Filter_Entry *dst);
  void SendMessage(Message *msg, u_int16_t port);

  // Timer functions
  void NeighborsTimeOut();
  void FilterTimeOut();
  void EnergyTimeOut();
};

#endif












