// *********************************************************
//
// header.hh     : Diffusion Datagram Header
// authors       : Chalermek Intanagonwiwat and Fabio Silva
//
// $Id: header.hh,v 1.9 2000/12/16 01:16:46 fabio Exp $
//
// *********************************************************

#ifndef header_hh
#define header_hh

typedef int *Packet;

#define MAX_PKT_SIZE       8192
#define BROADCAST_ADDR     0xFFFFFFFF
#define LOCALHOST_ADDR     0xFFFFFFFE
#define DIFFUSION_VERSION  2
#define DIFFUSION_PORT     2000

// For accessing fields in the diffusion header
#define LAST_HOP(x)     (x)->last_hop
#define NEXT_HOP(x)     (x)->next_hop
#define VERSION(x)      (x)->version
#define MSG_TYPE(x)     (x)->msg_type
#define DATA_LEN(x)     (x)->data_len
#define PKT_NUM(x)      (x)->pkt_num
#define RDM_ID(x)       (x)->rdm_id
#define NUM_ATTR(x)     (x)->num_attr
#define SRC_PORT(x)     (x)->src_port

// Message types
typedef enum msg_t_ {
  INTEREST,
  POSITIVE_REINFORCEMENT,
  NEGATIVE_REINFORCEMENT,
  DATA,
  FLAGGED_DATA,
  CONTROL,
  REDIRECT,
} diff_msg_t;

// Header structure
struct hdr_diff {
  int32_t         last_hop;           // forwarder node id
  int32_t         next_hop;           // next hop node id
  int8_t          version;            // protocol version
  int8_t          msg_type;           // message type
  int16_t         data_len;           // data length
  int32_t         pkt_num;            // packet number
  int32_t         rdm_id;             // random id
  int16_t         num_attr;           // number of attributes
  int16_t         src_port;           // sender port id
};

// For mapping attributes
class Packed_Attribute {
public:
  int32_t key;
  int8_t type;
  int8_t op;
  int16_t len;
  int32_t val;
};

#endif
