// *********************************************************
// 
// message.hh    : Message definitions
// authors       : Fabio Silva
//
// $Id: message.hh,v 1.15 2001/03/23 20:56:31 fabio Exp $
//
// *********************************************************

#ifndef message_hh
#define message_hh

#include "nr.hh"
#include "header.hh"
#include "attrs.hh"

class Message {
public:
  // Read directly from the packet header
  int8_t  version;
  int8_t  msg_type;
  u_int16_t source_port;
  int16_t data_len;
  int16_t num_attr;
  int32_t pkt_num;
  int32_t rdm_id;
  int32_t next_hop;
  int32_t last_hop;

  // Entire message
  Packet  msg;
  int32_t msg_size;

  // Other flags
  int new_message;

  // Message attributes
  NRAttrVec *msg_attr_vec;

  Message(int8_t _version, int8_t _msg_type, u_int16_t _source_port,
	  u_int16_t _data_len, u_int16_t _num_attr, int32_t _pkt_num,
	  int32_t _rdm_id, int32_t _next_hop, int32_t _last_hop) :
    version(_version),
    msg_type(_msg_type),
    source_port(_source_port),
    data_len(_data_len),
    num_attr(_num_attr),
    pkt_num(_pkt_num),
    rdm_id(_rdm_id),
    next_hop(_next_hop),
    last_hop(_last_hop)
  {
    msg = NULL;
    msg_attr_vec = NULL;
    new_message = 1;             // New message by default, will be changed
                                 // later if message is found to be old
  }

  ~Message()
  {
    if (msg)
      delete [] msg;

    if (msg_attr_vec){
      ClearAttrs(msg_attr_vec);
      delete msg_attr_vec;
    }
  }
};

extern NRSimpleAttributeFactory<void *> ControlMsgAttr;
extern NRSimpleAttributeFactory<void *> OriginalHdrAttr;
extern NRSimpleAttributeFactory<int> NRScopeAttr;
extern NRSimpleAttributeFactory<int> NRClassAttr;
extern NRSimpleAttributeFactory<int> NRNewSubsAttr;
extern NRSimpleAttributeFactory<void *> ReinforcementAttr;

#define CONTROL_MESSAGE_KEY  2400
#define REINFORCEMENT_KEY    2401
#define ORIGINAL_HEADER_KEY  2402

// Control Message types
typedef enum ctl_t_ {
  ADD_FILTER,
  REMOVE_FILTER,
  KEEPALIVE_FILTER,
  SEND_MESSAGE,
  SEND_TO_NEXT
} ctl_t;

class ControlMessage {
public:
  int32_t command;
  int32_t param1;
  int32_t param2;
};

class RedirectMessage {
public:
  int8_t  new_message;
  int8_t  msg_type;
  u_int16_t source_port;
  u_int16_t data_len;
  u_int16_t num_attr;
  int32_t rdm_id;
  int32_t pkt_num;
  int32_t next_hop;
  int32_t last_hop;
  int32_t handle;
};

Message * CopyMessage(Message *msg);

#endif
