// *********************************************************
//
// replicator.cc : Wired Support for Diffusion
// Authors       : Chalermek Intanagonwiwat and Fabio Silva
//
// $Id: replicator.cc,v 1.8 2001/01/22 19:56:35 fabio Exp $
//
// *********************************************************

#include <stdio.h>
#include <stdlib.h>

#include "dgram.hh"
#include "replicator.hh"
#include "header.hh"

DiffusionWired::DiffusionWired(char *config_file)
{
  Topology_Entry *entry;
  FILE *cfile;
  int flag, q;
  struct socket_info *skt;
  char hname[50];
  struct hostent *hent;
  
  srand(time(NULL));

  // Try to open the config file
  cfile = fopen(config_file, "r");
   
  if (cfile == NULL){
    printf("Can't find configuration file !\n");
    exit(-1);
  }

  // Parse the config file
  num_neighbors = 1;
  q = 0;
  printf("Parsing config file - %s \n", config_file);
  flag = fscanf(cfile, "%s - %d\n", hname, &q);
  while (flag != EOF){
    if (flag == 0){
      printf("Error in the config file !\n");
      exit(-1);
    }

    // Link quality not specified
    if (flag == 1){
      q = RELIABLE_LINK;
    }
    else{
      if (q < 0 || q > RELIABLE_LINK){
	printf("Error parsing config file: Link quality outside range (0-100) !\n");
	exit(-1);
      }
    }

    printf("Neighbor %d - %s - Link Quality = %d\n", num_neighbors, hname, q);
    hent = gethostbyname(hname);
    if (hent == NULL){
      printf("Can't resolve host name !\n");
      exit(-1);
    }
    
    skt = CreateSocket(hent, DIFFUSION_PORT);
    if (!skt){
      printf("Can't create socket to neighbor !\n");
      exit(-1);
    }

    entry = new Topology_Entry;
    entry->skt = skt;
    entry->quality = q;

    neighbors_list.push_back(entry);
    num_neighbors++;

    flag = fscanf(cfile, "%s - %d\n", hname, &q);
   }

  fclose(cfile);

  printf("\nTotal Neighbors = %d\n", (num_neighbors - 1));
}

void DiffusionWired::send(void *pkt, int len)
{
  TopologyList::iterator itr;
  Topology_Entry *entry;
  int prob_loss;

  for (itr = neighbors_list.begin(); itr != neighbors_list.end(); ++itr){
    entry = *itr;
    if (entry->quality == RELIABLE_LINK){
      SEND_TO(entry->skt, pkt);
    }
    else{
      prob_loss = (int) (RELIABLE_LINK * (rand() * 1.0 / RAND_MAX));
      if (prob_loss <= entry->quality)
	SEND_TO(entry->skt, pkt);
      else
	printf("Packet dropped !\n");
    }
  }
}
