// *********************************************************
// 
// stats.hh      : Collect various statistics for Diffusion
// authors       : Chalermek Intanagonwiwat and Fabio Silva
//
// $Id: stats.hh,v 1.1 2001/03/29 02:33:10 fabio Exp $
//
// *********************************************************

#ifndef stats_hh
#define stats_hh

#include <sys/time.h>
#include <unistd.h>
#include <list>

#include "message.hh"

class Neighbor_Stats_Entry {
public:
  int id;
  int recv_messages;
  int recv_bcast_messages;
  int sent_messages;

  Neighbor_Stats_Entry(int _id) : id(_id)
  {
    recv_messages = 0;
    recv_bcast_messages = 0;
    sent_messages = 0;
  }
};

typedef list<Neighbor_Stats_Entry *> NeighborStatsList;

class DiffusionStats {
public:
  DiffusionStats(int id);
  void logIncomingMessage(Message *msg);
  void logOutgoingMessage(Message *msg);
  void printStats(FILE *output);

private:
  int node_id;
  struct timeval start;
  struct timeval finish;
  NeighborStatsList nodes;

  // Counters
  int num_bytes_recv;
  int num_bytes_sent;
  int num_packets_recv;
  int num_packets_sent;
  int num_bcast_bytes_recv;
  int num_bcast_bytes_sent;
  int num_bcast_packets_recv;
  int num_bcast_packets_sent;

  // Message counter
  int num_interest_messages_sent;
  int num_interest_messages_recv;
  int num_data_messages_sent;
  int num_data_messages_recv;
  int num_flagged_data_messages_sent;
  int num_flagged_data_messages_recv;
  int num_pos_reinforcement_messages_sent;
  int num_pos_reinforcement_messages_recv;
  int num_neg_reinforcement_messages_sent;
  int num_neg_reinforcement_messages_recv;

  int num_new_messages_sent;
  int num_new_messages_recv;
  int num_old_messages_sent;
  int num_old_messages_recv;

  Neighbor_Stats_Entry * getNeighbor(int id);
};





#endif
